import json
import os
import platform
import sys

from .settings import Settings

class Utility:
    """
    The Utility object is shared amongst all parts of openglass.
    """

    def __init__(self, verbose=False):
        self.verbose = verbose

        # The platform Openglass is running on
        self.platform = platform.system()
        if self.platform.endswith("BSD") or self.platform == "DragonFly":
            self.platform = "BSD"

        with open(self.get_resource_path("version.txt")) as f:
            self.version = f.read().strip()

    def load_settings(self, config=None):
        """
        Loading settings, optionally from a custom config json file.
        """
        self.settings = Settings(self, config)
        self.settings.load()

    def print_settings(self, config=None):
        self.settings = Settings(self, config)
        self.settings.load()
        return self.settings.print_settings()

    def get_setting(self, value):
        return self.settings.get(value)

    def get_resource_path(self, filename):
        """
        Returns the absolute path of a resource, regardless of whether Openglass is installed
        systemwide, and whether regardless of platform
        """
        prefix = "share"

        # On Windows, and in Windows dev mode, switch slashes in incoming filename to backslackes
        if self.platform == "Windows":
            filename = filename.replace("/", "\\")

        if getattr(sys, "openglass_test_mode", False):
            # Look for resources directory relative to python file
            prefix = os.path.join(
                os.path.dirname(
                    os.path.dirname(
                        os.path.abspath(inspect.getfile(inspect.currentframe()))
                    )
                ),
                "share",
            )
            if not os.path.exists(prefix):
                # While running tests during stdeb bdist_deb, look 3 directories up for the share folder
                prefix = os.path.join(
                    os.path.dirname(
                        os.path.dirname(os.path.dirname(os.path.dirname(prefix)))
                    ),
                    "share",
                )

        elif self.platform == "BSD" or self.platform == "Linux":
            prefix = os.path.join(
                os.path.dirname(os.path.dirname(sys.argv[0])), "share/openglass"
            )

        elif getattr(sys, "openglass_test_mode", False):
            if self.platform == "Darwin":
                prefix = os.path.join(sys._MEIPASS, "share")
            elif self.platform == "Windows":
                prefix = os.path.join(os.path.dirname(sys.executable), "share")

        return os.path.join(prefix, filename)

    def build_data_dir(self):
        """
        Returns the path of the openglass data directory.
        """
        if self.platform == "Windows":
            try:
                appdata = os.environ["APPDATA"]
                openglass_data_dir = f"{appdata}\\openglass"
            except:
                # If for some reason we don't have the 'APPDATA' environment variable
                # (like running tests in Linux while pretending to be in Windows)
                openglass_data_dir = os.path.expanduser("~/.config/openglass")
        elif self.platform == "Darwin":
            openglass_data_dir = os.path.expanduser(
                "~/Library/Application Support/openglass"
            )
        else:
            openglass_data_dir = os.path.expanduser("~/.config/openglass")

        # Modify the data dir if running tests
        if getattr(sys, "openglass_test_mode", False):
            openglass_data_dir += "-testdata"

        os.makedirs(openglass_data_dir, 0o700, True)
        return openglass_data_dir

    def build_tmp_dir(self):
        """
        Returns path to a folder that can hold temporary files
        """
        tmp_dir = os.path.join(self.build_data_dir(), "tmp")
        os.makedirs(tmp_dir, 0o700, True)
        return tmp_dir

    def build_persistent_dir(self):
        """
        Returns the path to the folder that holds persistent files
        """
        persistent_dir = os.path.join(self.build_data_dir(), "persistent")
        os.makedirs(persistent_dir, 0o700, True)
        return persistent_dir

    def log(self, module, func, msg=None):
        """
        If verbose mode is on, log error messages to stdout
        """
        if self.verbose:
            timestamp = time.strftime("%b %d %Y %X")

            final_msg = f"[{timestamp}] {module}.{func}"
            if msg:
                final_msg = f"{final_msg}: {msg}"
            print(final_msg)



import json
import time
import tweepy

class Twitter:

    def __init__(self, twitter_apis):
        self.twitter_apis = twitter_apis
        self.api = self.__authenticate(self.twitter_apis[0])
        self.api_index = 0

    def get_followers(self, user):
        followers = []

        try:
            followers = self.api.followers(user)
        except tweepy.RateLimitError as e:
            self.__handle_time_limit()
        except Exception as e:
            return e

        return followers

    def get_profile(self, user):
        profile = []

        try:
            profile = self.api.get_user(user_id)
        except tweepy.RateLimitError as e:
            self.__handle_time_limit()
        except Exception as e:
            return e

        return profile

    def get_statuses(self, user):
        tweets = []
        try:
            tweets = self.api.user_timeline(user, count=200)
        except tweepy.RateLimitError as e:
            self.__handle_time_limit()
        except Exception as e:
            return e

        return tweets

    def search(self, q):
        tweets = []

        try:
            tweets = self.api.search(q, count=500)
        except tweepy.RateLimitError as e:
            self.__handle_time_limit()
        except Exception as e:
            return e

        return tweets

    def __authenticate(self, credentials):
            auth = tweepy.OAuthHandler(credentials['CONSUMER_KEY'], credentials['CONSUMER_SECRET'])
            auth.set_access_token(credentials['ACCESS_KEY'], credentials['ACCESS_SECRET'])
            api = tweepy.API(auth, parser=tweepy.parsers.JSONParser())
            return api

    def __limit_handled(self, cursor):
        while True:
            try:
                yield cursor.next()
            except tweepy.RateLimitError:
                self.__handle_time_limit()

    def __handle_time_limit(self):
        if self.twitter_apis.len() > 1:
            if self.api_index == (self.twitter_apis.len() - 1):
                window = 15 / self.twitter_apis.len()
                time.sleep(window * 60)
            else:
                self.api_index += 1
                self.api = self.__authenticate(self.twitter_apis[self.api_index])
        else:
            time.sleep(15 * 60)



import json
import telethon.sync

from telethon import TelegramClient
from telethon.errors import SessionPasswordNeededError
from telethon.tl.functions.channels import GetParticipantsRequest
from telethon.tl.types import ChannelParticipantsSearch
from telethon.tl.functions.messages import (GetHistoryRequest)
from telethon.tl.types import (
    PeerChannel
)

class Telegram:

    def __init__(self, telegram_apis):
        self.telegram_apis = telegram_apis
        self.api =  self.__authenticate(self.telegram_apis)

    def get_channel(self, channel):
        client = self.api

        if channel.isdigit():
            entity = PeerChannel(int(channel))
        else:
            entity = channel

        my_channel = client.get_entity(entity)
        offset = 0
        limit = 100
        all_participants = []

        while True:
            participants = client(GetParticipantsRequest(
                my_channel, ChannelParticipantsSearch(''), offset, limit,
                hash=0
            ))
            if not participants.users:
                break
            all_participants.extend(participants.users)
            offset += len(participants.users)

        all_user_details = []
        for participant in all_participants:
            all_user_details.append(
                {
                    "id": participant.id,
                    "first_name": participant.first_name,
                    "last_name": participant.last_name,
                    "user": participant.username,
                    "phone": participant.phone,
                    "is_bot": participant.bot,
                    "contact": participant.contact,
                    "mutual_contact": participant.mutual_contact,
	            "scam": participant.scam
                }
            )

        return all_user_details

    def get_messages(self, channel):
        client = self.api

        if channel.isdigit():
            entity = PeerChannel(int(channel))
        else:
            entity = channel

        my_channel = client.get_entity(entity)

        offset_id = 0
        limit = 100
        all_messages = []
        total_messages = 0
        total_count_limit = 0

        while True:
            print("Current Offset ID is:", offset_id, "; Total Messages:", total_messages)
            history = client(GetHistoryRequest(
                peer=my_channel,
                offset_id=offset_id,
                offset_date=None,
                add_offset=0,
                limit=limit,
                max_id=0,
                min_id=0,
                hash=0
            ))
            if not history.messages:
                break
            messages = history.messages
            for message in messages:
                all_messages.append(message.to_dict())
            offset_id = messages[len(messages) - 1].id
            total_messages = len(all_messages)
            if total_count_limit != 0 and total_messages >= total_count_limit:
                break
        return all_messages

    def __authenticate(self, credentials):
        # Setting configuration values
        api_id = credentials['app_id']
        api_hash = credentials['api_hash']

        api_hash = str(api_hash)

        phone = credentials['phone']
        username = credentials['username']
        client = TelegramClient(username, api_id, api_hash)
        client.start()
        print("Client Created")
        # Ensure you're authorized
        if not client.is_user_authorized():
            client.send_code_request(phone)
            try:
                client.sign_in(phone, input('Enter the code: '))
            except SessionPasswordNeededError:
                client.sign_in(password=input('Password: '))

        return client



import argparse
import csv
from datetime import date, datetime
import json
import sys

from .twitter import Twitter
from .telegram import Telegram
from .utility import Utility

def main(cwd=None):
    """
    The main() function implements all of the logic that the command-line version of
    openglass uses.
    """

    utility = Utility()

    if utility.platform == "Darwin":
        if cwd:
            os.chdir(cwd)

    # Parse arguments
    parser = argparse.ArgumentParser(
        formatter_class=lambda prog: argparse.HelpFormatter(prog, max_help_position=28)
    )

    parser.add_argument(
        "--config",
        metavar="FILENAME",
        default=None,
        help="Filename of custom global settings",
    )
    parser.add_argument(
        "--settings",
        action='store_true',
        help="Print openglass setting",
    )
    parser.add_argument(
        "--version",
        action='store_true',
        help="Print openglass version",
    )
    parser.add_argument(
        "--csv",
        action='store_true',
        help="Print results as csv",
    )
    parser.add_argument(
        "--twitter",
        action='store_true',
        help="Query the twitter endpoints. Use search to search, timeline for user's statuses, profile for user's profile",
    )

    parser.add_argument(
        "--search",
        metavar="SEARCH QUERY",
        default=None,
        help="Specify the term to search",
    )

    parser.add_argument(
        "--timeline",
        metavar="USERNAME OR ID",
        default=None,
        help="Specify the user to retrieve its timeline",
    )

    parser.add_argument(
        "--profile",
        metavar="USERNAME OR ID",
        default=None,
        help="Specify the user to retrieve its profile",
    )

    parser.add_argument(
        "--followers",
        metavar="USERNAME OR ID",
        default=None,
        help="Specify the user to retrieve its profile",
    )

    parser.add_argument(
        "--telegram",
        action='store_true',
        help="Query telegram enpoints",
    )

    parser.add_argument(
        "--channel-users",
        metavar="CHANNEL ID",
        default=None,
        help="Query telegram channels and return its users in json",
    )

    parser.add_argument(
        "--channel-messages",
        metavar="CHANNEL ID",
        default=None,
        help="Query telegram channels and return its messages in json",
    )


    args = parser.parse_args()

    version = bool(args.version)
    settings = bool(args.settings)
    csv = bool(args.csv)
    twitter = bool(args.twitter)
    timeline = bool(args.timeline)
    q_timeline = args.timeline
    search = bool(args.search)
    q_search = args.search
    profile = bool(args.profile)
    q_profile = args.profile
    followers = bool(args.followers)
    q_followers = args.followers
    telegram = bool(args.telegram)
    channel_users = bool(args.channel_users)
    q_channel_users = args.channel_users
    channel_messages = bool(args.channel_messages)
    q_channel_messages = args.channel_messages
    config_filename = args.config

    if version:
        print(
            "Openglass version {}".format(utility.version)
        )
        sys.exit()

    if settings:
        if config_filename:
            print(
                "Openglass settings {}".format(utility.print_settings(config_filename))
            )
        else:
            print(
                "Openglass settings {}".format(utility.print_settings())
            )
        sys.exit()

    # Re-load settings, if a custom config was passed in
    if config_filename:
        utility.load_settings(config_filename)
    else:
        utility.load_settings()

    if twitter:
        if config_filename:
            utility.load_settings(config_filename)
        else:
            utility.load_settings()

        t = Twitter(utility.get_setting('twitter_apis'))
        if search:
            res = t.search(q_search)['statuses']
            if csv:
                save_as_csv(res, 'results.csv')
            else:
                print(json.dumps(res, indent=4, sort_keys=True))
            sys.exit()
        elif timeline:
            res = t.get_statuses(q_timeline)
            if csv:
                save_as_csv(res, 'results.csv')
            else:
                print(json.dumps(res, indent=4, sort_keys=True))
            sys.exit()
        elif profile:
            res = t.get_statuses(q_profile)
            if csv:
                save_as_csv(res, 'results.csv')
            else:
                print(json.dumps(res, indent=4, sort_keys=True))
            sys.exit()
        elif followers:
            res = t.get_followers(q_followers)
            if csv:
                save_as_csv(res, 'results.csv')
            else:
                print(json.dumps(res, indent=4, sort_keys=True))
            sys.exit()

    if telegram:
        if config_filename:
            utility.load_settings(config_filename)
        else:
            utility.load_settings()

        t = Telegram(utility.get_setting('telegram'))

        if channel_users:
            res = t.get_channel(q_channel_users)
            if csv:
                save_as_csv(res, 'results.csv')
            else:
                print(json.dumps(res, indent=4, sort_keys=True))
            sys.exit()
        elif channel_messages:
            res = t.get_messages(q_channel_messages)
            if search:
                res = search_dict(res, q_search)
            if csv:
                save_as_csv(res, 'results.csv')
            else:
                print(json.dumps(res, indent=4, sort_keys=True, cls=DateTimeEncoder))
            sys.exit()

def save_as_csv(res_dict, csvfile):
    """
    Takes a list of dictionaries as input and outputs a CSV file.
    """
    with open(csvfile, 'w', newline='') as csvfile:
        fieldnames = res_dict[0].keys()
        writer = csv.DictWriter(csvfile, fieldnames=fieldnames,
                                         extrasaction='ignore', delimiter = ';')
        writer.writeheader()
        for r in res_dict:
            writer.writerow(r)

def search_dict(res_dict, query_value):
    filtered_dict = []
    for r in res_dict:
        path = getpath(r, query_value)
        if path:
            filtered_dict.append(r)
    return filtered_dict

def getpath(nested_dict, value, prepath=()):
    for k, v in nested_dict.items():
        path = prepath + (k,)
        if value in str(v): # found value
            return path
        elif hasattr(v, 'items'): # v is a dict
            p = getpath(v, value, path) # recursive call
            if p is not None:
                return p

# some functions to parse json date
class DateTimeEncoder(json.JSONEncoder):
    def default(self, o):
        if isinstance(o, datetime):
            return o.isoformat()

        if isinstance(o, bytes):
            return list(o)

        return json.JSONEncoder.default(self, o)

if __name__ == "__main__":
    main()



import json
import os
import platform

try:
    import pwd
except:
    pass

class Settings(object):
    """
    This class stores all of the settings for openglass.
    """

    def __init__(self, utility, config=False):
        self.utility = utility

        self.utility.log("Settings", "__init__")

        # If a readable config file was provided, use that instead
        if config:
            if os.path.isfile(config):
                self.filename = config
            else:
                self.utility.log(
                    "Settings",
                    "__init__",
                    "Supplied config does not exist or is unreadable. Falling back to default location",
                )
                self.filename = self.build_filename()

        else:
            # Default config
            self.filename = self.build_filename()

        # These are the default settings. They will get overwritten when loading from disk
        self.default_settings = {
            "version": self.utility.version,
            "twitter_apis": [],
        }
        self._settings = {}
        self.fill_in_defaults()

    def fill_in_defaults(self):
        """
        If there are any missing settings from self._settings, replace them with
        their default values.
        """
        for key in self.default_settings:
            if key not in self._settings:
                self._settings[key] = self.default_settings[key]

    def build_filename(self):
        """
        Returns the path of the settings file.
        """
        return os.path.join(self.utility.build_data_dir(), "config.json")

    def print_settings(self):
        return json.dumps(self._settings, indent=4, sort_keys=True)

    def load(self):
        """
        Load the settings from file.
        """
        self.utility.log("Settings", "load")

        # If the settings file exists, load it
        if os.path.exists(self.filename):
            try:
                self.utility.log("Settings", "load", f"Trying to load {self.filename}")
                with open(self.filename, "r") as f:
                    self._settings = json.loads(f.read())
                    self.fill_in_defaults()
            except:
                pass

    def save(self):
        """
        Save settings to file.
        """
        self.utility.log("Settings", "save")
        open(self.filename, "w").write(json.dumps(self._settings, indent=2))
        self.utility.log("Settings", "save", f"Settings saved in {self.filename}")

    def get(self, key):
        return self._settings[key]

    def set(self, key, val):
        self._settings[key] = val
